@@ -1,3 +1,5 @@
+import math
+from math import sqrt
 from functools import partial
 import torch
 from torch import nn, einsum
@@ -96,6 +98,9 @@ def stable_softmax(t, dim = -1, alpha = 32 ** 2):
 def prob_mask_like(shape, prob, device):
     return torch.zeros(shape, device = device).float().uniform_(0, 1) < prob
 
+def l2norm(t):
+    return F.normalize(t, dim = -1)
+
 # gan losses
 
 def hinge_discr_loss(fake, real):
@@ -172,6 +177,40 @@ class Discriminator(nn.Module):
 
         return self.to_logits(x)
 
+class ContinuousPositionBias(nn.Module):
+    """ from https://arxiv.org/abs/2111.09883 """
+
+    def __init__(self, *, dim, heads, layers = 2):
+        super().__init__()
+        self.net = MList([])
+        self.net.append(nn.Sequential(nn.Linear(2, dim), nn.ReLU()))
+
+        for _ in range(layers - 1):
+            self.net.append(nn.Sequential(nn.Linear(dim, dim), nn.ReLU()))
+
+        self.net.append(nn.Linear(dim, heads))
+        self.register_buffer('rel_pos', None, persistent = False)
+
+    def forward(self, x):
+        n, device = x.shape[-1], x.device
+        fmap_size = int(sqrt(n))
+
+        if not exists(self.rel_pos):
+            pos = torch.arange(fmap_size, device = device)
+            grid = torch.stack(torch.meshgrid(pos, pos, indexing = 'ij'))
+            grid = rearrange(grid, 'c i j -> (i j) c')
+            rel_pos = rearrange(grid, 'i c -> i 1 c') - rearrange(grid, 'j c -> 1 j c')
+            rel_pos = torch.sign(rel_pos) * torch.log(rel_pos.abs() + 1)
+            self.register_buffer('rel_pos', rel_pos, persistent = False)
+
+        rel_pos = self.rel_pos.float()
+
+        for layer in self.net:
+            rel_pos = layer(rel_pos)
+
+        bias = rearrange(rel_pos, 'i j h -> h i j')
+        return x + bias
+
 class ResBlock(nn.Module):
     def __init__(self, chan, groups = 32):
         super().__init__()
@@ -194,14 +233,18 @@ class VQGanAttention(nn.Module):
         *,
         dim,
         dim_head = 64,
-        heads = 8
+        heads = 8,
+        dropout = 0.
     ):
         super().__init__()
         self.heads = heads
-        self.scale = dim_head ** -0.5
+        self.scale = nn.Parameter(torch.ones(1, heads, 1, 1) * math.log(0.01))
         inner_dim = heads * dim_head
 
-        self.norm = LayerNormChan(dim)
+        self.dropout = nn.Dropout(dropout)
+        self.post_norm = LayerNormChan(dim)
+
+        self.cpb = ContinuousPositionBias(dim = dim // 4, heads = heads)
         self.to_qkv = nn.Conv2d(dim, inner_dim * 3, 1, bias = False)
         self.to_out = nn.Conv2d(inner_dim, dim, 1)
 
@@ -209,20 +252,24 @@ class VQGanAttention(nn.Module):
         h = self.heads
         height, width, residual = *x.shape[-2:], x.clone()
 
-        x = self.norm(x)
-
         q, k, v = self.to_qkv(x).chunk(3, dim = 1)
 
         q, k, v = map(lambda t: rearrange(t, 'b (h c) x y -> b h c (x y)', h = h), (q, k, v))
-        sim = einsum('b h c i, b h c j -> b h i j', q, k) * self.scale
+
+        q, k = map(l2norm, (q, k))
+
+        sim = einsum('b h c i, b h c j -> b h i j', q, k) * self.scale.exp()
+
+        sim = self.cpb(sim)
 
         attn = stable_softmax(sim, dim = -1)
+        attn = self.dropout(attn)
 
         out = einsum('b h i j, b h c j -> b h c i', attn, v)
         out = rearrange(out, 'b h c (x y) -> b (h c) x y', x = height, y = width)
         out = self.to_out(out)
 
-        return out + residual
+        return self.post_norm(out) + residual
 
 class VQGanVAE(nn.Module):
     def __init__(
@@ -246,6 +293,7 @@ class VQGanVAE(nn.Module):
         attn_dim_head = 64,
         attn_heads = 8,
         resnet_groups = 32,
+        attn_dropout = 0.,
         **kwargs
     ):
         super().__init__()
@@ -270,24 +318,36 @@ class VQGanVAE(nn.Module):
         dims = (dim, *layer_dims)
         codebook_dim = layer_dims[-1]
 
-        reversed_dims = tuple(reversed(dims))
-        enc_dim_pairs = zip(dims[:-1], dims[1:])
-        dec_dim_pairs = zip(reversed_dims[:-1], reversed_dims[1:])
+        dim_pairs = zip(dims[:-1], dims[1:])
 
-        for _, (enc_dim_in, enc_dim_out), (dec_dim_in, dec_dim_out) in zip(range(num_layers), enc_dim_pairs, dec_dim_pairs):
-            self.encoders.append(nn.Sequential(nn.Conv2d(enc_dim_in, enc_dim_out, 4, stride = 2, padding = 1), nn.ReLU()))
-            self.decoders.append(nn.Sequential(nn.Upsample(scale_factor = 2, mode = 'bilinear', align_corners = False), nn.Conv2d(dec_dim_in, dec_dim_out, 3, padding = 1), nn.ReLU()))
+        append = lambda arr, t: arr.append(t)
+        prepend = lambda arr, t: arr.insert(0, t)
 
-        for _ in range(num_resnet_blocks):
-            self.encoders.append(ResBlock(dims[-1], groups = resnet_groups))
-            self.decoders.insert(0, ResBlock(dims[-1], groups = resnet_groups))
+        if not isinstance(num_resnet_blocks, tuple):
+            num_resnet_blocks = (*((0,) * (num_layers - 1)), num_resnet_blocks)
 
-        if use_attn:
-            self.encoders.append(VQGanAttention(dim = dims[-1], heads = attn_heads, dim_head = attn_dim_head))
-            self.decoders.insert(0, VQGanAttention(dim = dims[-1], heads = attn_heads, dim_head = attn_dim_head))
+        if not isinstance(use_attn, tuple):
+            use_attn = (*((False,) * (num_layers - 1)), use_attn)
 
-        self.encoders.insert(0, nn.Conv2d(channels, dim, channels, padding = 1))
-        self.decoders.append(nn.Conv2d(dim, channels, 1))
+        assert len(num_resnet_blocks) == num_layers, 'number of resnet blocks config must be equal to number of layers'
+        assert len(use_attn) == num_layers
+
+        for layer_index, (dim_in, dim_out), layer_num_resnet_blocks, layer_use_attn in zip(range(num_layers), dim_pairs, num_resnet_blocks, use_attn):
+            append(self.encoders, nn.Sequential(nn.Conv2d(dim_in, dim_out, 4, stride = 2, padding = 1), nn.ReLU()))
+            prepend(self.decoders, nn.Sequential(nn.Upsample(scale_factor = 2, mode = 'bilinear', align_corners = False), nn.Conv2d(dim_out, dim_in, 3, padding = 1), nn.ReLU()))
+
+            if layer_use_attn:
+                prepend(self.decoders, VQGanAttention(dim = dims[-1], heads = attn_heads, dim_head = attn_dim_head, dropout = attn_dropout))
+
+            for _ in range(layer_num_resnet_blocks):
+                append(self.encoders, ResBlock(dim_out, groups = resnet_groups))
+                prepend(self.decoders, ResBlock(dim_out, groups = resnet_groups))
+
+            if layer_use_attn:
+                append(self.encoders, VQGanAttention(dim = dims[-1], heads = attn_heads, dim_head = attn_dim_head, dropout = attn_dropout))
+
+        prepend(self.encoders, nn.Conv2d(channels, dim, 3, padding = 1))
+        append(self.decoders, nn.Conv2d(dim, channels, 1))
 
         self.vq = VQ(
             dim = codebook_dim,
