@@ -1,4 +1,5 @@
 from math import sqrt
+from random import choice
 from pathlib import Path
 from shutil import rmtree
 
@@ -58,6 +59,9 @@ def get_optimizer(
     if filter_by_requires_grad:
         params = list(filter(lambda t: t.requires_grad, params))
 
+    if wd == 0:
+        return Adam(params, lr = lr)
+
     params = set(params)
     wd_params, no_wd_params = separate_weight_decayable_params(params)
 
@@ -75,17 +79,18 @@ class MemmappedImageDataset(Dataset):
         self,
         *,
         path,
-        shape
+        shape,
+        random_rotate = True
     ):
         super().__init__()
         path = Path(path)
         assert path.exists(), f'path {path} must exist'
         self.memmap = np.memmap(str(path), mode = 'r', dtype = np.uint8, shape = shape)
+        self.random_rotate = random_rotate
 
         image_size = shape[-1]
         self.transform = T.Compose([
             T.Resize(image_size),
-            T.RandomHorizontalFlip(),
             T.CenterCrop(image_size),
             T.ToTensor()
         ])
@@ -100,7 +105,11 @@ class MemmappedImageDataset(Dataset):
             arr = rearrange(arr, '1 ... -> ...')
 
         img = Image.fromarray(arr)
-        return self.transform(img)
+        img = self.transform(img)
+
+        if self.random_rotate:
+            img = T.functional.rotate(img, choice([0, 90, 180, 270]))
+        return img
 
 class ImageDataset(Dataset):
     def __init__(
@@ -141,6 +150,7 @@ class VQGanVAETrainer(nn.Module):
         lr,
         batch_size,
         grad_accum_every,
+        wd = 0.,
         images_memmap_path = None,
         images_memmap_shape = None,
         folder = None,
@@ -158,7 +168,7 @@ class VQGanVAETrainer(nn.Module):
         self.batch_size = batch_size
         self.grad_accum_every = grad_accum_every
 
-        self.optim = Adam(vae.parameters(), lr = lr)
+        self.optim = get_optimizer(vae.parameters(), lr = lr, wd = wd)
 
         # create dataset
 
